/**
 * WEBLAB: Service oriented integration platform for media mining and intelligence applications
 * 
 * Copyright (C) 2004 - 2012 Cassidian, an EADS company
 * 
 * This library is free software; you can redistribute it and/or modify it under the terms of
 * the GNU Lesser General Public License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License along with this
 * library; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA 02110-1301 USA
 */

package org.ow2.weblab.service.boilerpipe;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;

import javax.jws.WebService;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.ow2.weblab.content.api.ContentManager;
import org.ow2.weblab.core.extended.exception.WebLabCheckedException;
import org.ow2.weblab.core.extended.factory.MediaUnitFactory;
import org.ow2.weblab.core.model.Document;
import org.ow2.weblab.core.model.Resource;
import org.ow2.weblab.core.model.Text;
import org.ow2.weblab.core.services.Analyser;
import org.ow2.weblab.core.services.InsufficientResourcesException;
import org.ow2.weblab.core.services.InvalidParameterException;
import org.ow2.weblab.core.services.UnexpectedException;
import org.ow2.weblab.core.services.analyser.ProcessArgs;
import org.ow2.weblab.core.services.analyser.ProcessReturn;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

import de.l3s.boilerpipe.BoilerpipeProcessingException;
import de.l3s.boilerpipe.document.TextDocument;
import de.l3s.boilerpipe.extractors.ArticleExtractor;
import de.l3s.boilerpipe.filters.heuristics.DocumentTitleMatchClassifier;
import de.l3s.boilerpipe.sax.BoilerpipeHTMLParser;

/**
 * This class is a WebService using the boilerpipe API in order to extract text
 * from HTML documents
 * 
 * @author khelif, nmartin
 */

@WebService(endpointInterface = "org.ow2.weblab.core.services.Analyser")
public class BoilerPipeHTMLPurificationService implements Analyser {


	final protected ContentManager contentManager;


	final protected Log log;


	public BoilerPipeHTMLPurificationService() {
		this.contentManager = ContentManager.getInstance();
		this.log = LogFactory.getLog(this.getClass());
	}


	@Override
	public ProcessReturn process(final ProcessArgs args) throws UnexpectedException, InsufficientResourcesException, InvalidParameterException {

		if (args == null) {
			throw new InsufficientResourcesException("ProcessArgs was null.", InsufficientResourcesException.class.getName());
		}
		final Resource res = args.getResource();
		if (res == null) {
			throw new InsufficientResourcesException("Resource of ProcessArgs was null.", InsufficientResourcesException.class.getName());
		}
		if (!(res instanceof Document)) {
			throw new InvalidParameterException("Invalid parameter: Resource of ProcessArgs was not a Document, but a " + res.getClass().getName(),
					InvalidParameterException.class.getName());
		}
		final Document doc = (Document) res;

		File file;
		try {
			file = this.contentManager.readNativeContent(doc);
		} catch (final WebLabCheckedException wlce) {
			throw new UnexpectedException("Content not available", UnexpectedException.class.getName(), wlce);
		}

		FileReader fd;
		try {
			fd = new FileReader(file);
		} catch (final FileNotFoundException e) {
			throw new UnexpectedException("Content not available", UnexpectedException.class.getName(), e);
		}

		final BoilerpipeHTMLParser parser = new BoilerpipeHTMLParser();
		try {
			parser.parse(new InputSource(fd));
		} catch (final SAXException e) {
			throw new UnexpectedException("Content not available", UnexpectedException.class.getName(), e);
		} catch (final IOException e) {
			throw new UnexpectedException("Content not available", UnexpectedException.class.getName(), e);
		} finally {
			try {
				fd.close();
			} catch (final IOException e) {
				throw new UnexpectedException("Could not close the stream", UnexpectedException.class.getName(), e);
			}
		}

		final TextDocument textDocument = parser.toTextDocument();
		final DocumentTitleMatchClassifier documentTitleMatchClassifier = new DocumentTitleMatchClassifier(textDocument.getTitle());

		try {
			documentTitleMatchClassifier.process(textDocument);
		} catch (final BoilerpipeProcessingException e2) {
			this.log.warn("Could not transform blocks containing parts of the HTML <TITLE> tag as content blocks.", e2);
		}

		try {
			ArticleExtractor.INSTANCE.process(textDocument);
		} catch (final BoilerpipeProcessingException e) {
			throw new UnexpectedException("Problem while trying to mark TextDocument using ArticleExtractor. Stopping the process...",
					UnexpectedException.class.getName(), e);
		}


		/*
		 * Retrieve both content and non-content
		 */
		final String normalisedText = textDocument.getText(true, false);

		final ProcessReturn pr = new ProcessReturn();
		if (normalisedText.length() <= 0) {
			this.log.warn("File was not a HTML file or was empty. No content appened.");
			pr.setResource(doc);
			return pr;
		}

		final Text text = MediaUnitFactory.createAndLinkMediaUnit(doc, Text.class);

		text.setContent(textDocument.getText(true, false));

		pr.setResource(doc);
		return pr;
	}

}
